# generator.py - generate output files

# Copyright (c) 2007-2008, Jason Breti (http://breti.org/codegen)
# All rights reserved.
# This code is released under the BSD License, as specified in the
# license.txt file.

import os, sys

from parser import Parser

class Generator:
  def generateChart(self, stateChartXML, fileName, template):
    parser = Parser()

    template.stateChartXML = stateChartXML
    template.userCode      = parser.parseInputFile(fileName)

    self.generateOutputFile(fileName, template)

  def generateState(self, stateChartXML, state, fileName, template):
    parser = Parser()

    template.state         = state
    template.stateChartXML = stateChartXML
    template.userCode      = parser.parseInputFile(fileName)
    template.searchPrefix  = parser.searchPrefix
    
    self.generateOutputFile(fileName, template)

  def generateOutputFile(self, fileName, template):
    backupFileName = self.getBackupFileName(fileName)
    if os.path.exists(fileName):
      os.rename(fileName, backupFileName)
    
    outputFile = open(fileName,'w')
    try:
      outputFile.write(str(template))
    finally:
      outputFile.close()

  def getBackupFileName(self, fileName):
    startVersion = 1
    backupFileName = fileName + '.backup.' + '%03d'%startVersion
    while os.path.isfile(backupFileName):
      startVersion += 1
      backupFileName = fileName + '.backup.' + '%03d'%startVersion

    return backupFileName

