# parser.py - Parse Input Files

# Copyright (c) 2007-2008, Jason Breti (http://breti.org/codegen)
# All rights reserved.
# This code is released under the BSD License, as specified in the
# license.txt file.

import os

class Parser:
  searchPrefix = '//#-#--HandCraftedCode'

  def parseInputFile(self, fileName):
    class InvalidMarker(AttributeError):
      "Used to indicate a mismatch between markers during parsing"
      
    searchMarker = {'begin': 'Begin',
                     'end'  : 'End'}
    userCode = { }
    
    if os.path.isfile(fileName):
      inputFile = open(fileName,'r')
      try:
        saveLines = False
        parsingMarker = 'unknown'
        
        lineCount = 0
    
        for line in inputFile:
          lineCount += 1
          if saveLines == False:
            if line.startswith(self.searchPrefix + ' End'):
              saveLines = False
              raise InvalidMarker, \
                "line "+str(lineCount)+\
                ": Unexpected end around "+parsingMarker+\
                " marker."
          else:
            if line.startswith(self.searchPrefix + ' End'):
              saveLines = False
            elif line.startswith(self.searchPrefix + ' Begin'):
              saveLines = False
              raise InvalidMarker, \
                "line "+str(lineCount)+\
                ": Unexpected begin while scanning "+parsingMarker+\
                " marker."
      
          if saveLines == True:
            tempLine = line.strip('\n')
            userCode.setdefault( parsingMarker, [ ] ).append(tempLine)
            
          if saveLines == False:
            fullPrefix = self.searchPrefix + ' Begin'
            if line.startswith(fullPrefix):
              parsingMarker = line[len(fullPrefix):].strip().split()[0]
              if userCode.has_key(parsingMarker):
                raise InvalidMarker, \
                  "line "+str(lineCount)+\
                  ": Duplicate marker "+parsingMarker+\
                  " found."
              else:
                saveLines = True
          
      finally:
        inputFile.close()
    
    return userCode
