//-------------------------------------------------------------------------
//
// File Name: sm_stateChart.cpp
//
// This file is code generated.  Do not make any changes to this file as
// they will be overwritten.
//
// Copyright (c) 2007-2008, Jason Breti (http://breti.org/codegen)
// All rights reserved.
// This code is released under the BSD License, as specified in the
// license.txt file.
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
#include "sm_stateChart.h"             // class SM_StateChart

#include <string>
#include <iostream>

// Include all the necessary states
#include "sm_state_Initial.h"
#include "sm_state_Unarmed.h"
#include "sm_state_Armed.h"
#include "sm_state_WaitingToUnarm.h"


//-------------------------------------------------------------------------
//
// Class Name : SM_StateChart
//
//-------------------------------------------------------------------------

SM_StateChart::SM_StateChart ()
:
  m_states(),
  m_transitions(),
  m_currentState()
{
}

//-------------------------------------------------------------------------
SM_StateChart::~SM_StateChart ()
{
  if (!m_states.empty())
  {
    States::iterator statesIterator;
  
    for ( statesIterator =  m_states.begin();
          statesIterator != m_states.end();
          ++statesIterator)
    {
      delete statesIterator->second;
    }
  }
}

//-------------------------------------------------------------------------
SM_State* SM_StateChart::initialize ()
{
  std::cout << "SM_StateChart::initialize\n";

  m_currentState = "Initial";
  
  SM_State* initialStatePtr = 0;
  SM_State* tempStatePtr = 0;
  
  // Construct all the necessary states
  
  // Add Initial State to the Map
  tempStatePtr = new SM_State_Initial;
  
  if (tempStatePtr != 0)
  {
    m_states.insert( StatePair("Initial", tempStatePtr) );
  }

  // Add Unarmed State to the Map
  tempStatePtr = new SM_State_Unarmed;
  
  if (tempStatePtr != 0)
  {
    m_states.insert( StatePair("Unarmed", tempStatePtr) );
  }

  // Add Armed State to the Map
  tempStatePtr = new SM_State_Armed;
  
  if (tempStatePtr != 0)
  {
    m_states.insert( StatePair("Armed", tempStatePtr) );
  }

  // Add WaitingToUnarm State to the Map
  tempStatePtr = new SM_State_WaitingToUnarm;
  
  if (tempStatePtr != 0)
  {
    m_states.insert( StatePair("WaitingToUnarm", tempStatePtr) );
  }


  // Connect the states to each other through transitions
  TransitionStep tempTransitionStep;

  // Add Transitions from the Initial state to all other permitted states
  tempTransitionStep.insert( TransitionStepPair(
                                SM_EVENT_BatteryOn,
                                "Unarmed")  );
  m_transitions.insert( TransitionsPair("Initial", tempTransitionStep) );

  tempTransitionStep.clear();

  // Add Transitions from the Unarmed state to all other permitted states
  tempTransitionStep.insert( TransitionStepPair(
                                SM_EVENT_RemoteArm,
                                "Armed")  );
  m_transitions.insert( TransitionsPair("Unarmed", tempTransitionStep) );

  tempTransitionStep.clear();

  // Add Transitions from the Armed state to all other permitted states
  tempTransitionStep.insert( TransitionStepPair(
                                SM_EVENT_RemoteUnarm,
                                "WaitingToUnarm")  );
  m_transitions.insert( TransitionsPair("Armed", tempTransitionStep) );

  tempTransitionStep.clear();

  // Add Transitions from the WaitingToUnarm state to all other permitted states
  tempTransitionStep.insert( TransitionStepPair(
                                SM_EVENT_DoorsOpen,
                                "Unarmed")  );
  tempTransitionStep.insert( TransitionStepPair(
                                SM_EVENT_UnarmTimerExpired,
                                "Armed")  );
  m_transitions.insert( TransitionsPair("WaitingToUnarm", tempTransitionStep) );

  tempTransitionStep.clear();

  return initialStatePtr;
}

//-------------------------------------------------------------------------
bool SM_StateChart::transitionState (SM_EVENTS transitionEvent,
                                     SM_State*& nextState)
{
  bool success = false;
  nextState = 0;
  
  // Find the map containing the transitions for the current state
  Transitions::iterator transitionsIterator;
  transitionsIterator = m_transitions.find(m_currentState);
  
  if (transitionsIterator != m_transitions.end())
  {
    // Inside those transitions find the next state, if a transition exists
    // based on this event
    TransitionStep::iterator transitionStepIterator;
    transitionStepIterator = transitionsIterator->second.find(transitionEvent);
    
    if (transitionStepIterator != transitionsIterator->second.end())
    {
      States::iterator statesIterator;
      
      statesIterator = m_states.find(transitionStepIterator->second);
      
      if (statesIterator != m_states.end())
      {
        nextState = statesIterator->second;
        m_currentState = transitionStepIterator->second;
        success = true;
      }
    }
  }
  
  return success;
}

//-------------------------------------------------------------------------
