//-------------------------------------------------------------------------
//
// File Name: sm_stateChart.h
//
// Copyright (c) 2007-2008, Jason Breti (http://breti.org/codegen)
// All rights reserved.
// This code is released under the BSD License, as specified in the
// license.txt file.
//-------------------------------------------------------------------------

#ifndef SM_STATECHART_H
#define SM_STATECHART_H

#include "sm_state.h"
#include "sm_events.h"
#include <map>
#include <string>

//-------------------------------------------------------------------------
class SM_StateChart
{
public:

  //-----------------------------------------------------------------------
  SM_StateChart ();

  //-----------------------------------------------------------------------
  ~SM_StateChart ();

  //-----------------------------------------------------------------------
  SM_State* initialize ();
  
  //-----------------------------------------------------------------------
  bool transitionState (SM_EVENTS transitionEvent, SM_State*& nextState);

protected:

private:
  //-----------------------------------------------------------------------
  // copy constructor not implemented
  SM_StateChart( const SM_StateChart& );

  // assignment operator not implemented
  SM_StateChart& operator=( const SM_StateChart& );
  
  //-----------------------------------------------------------------------
  // The following typedefs are used for the map that contains pointers
  // to the states.  All these objects should be freed to prevent memory
  // leaks.
  typedef std::map<std::string, SM_State*> 
          States;

  typedef std::pair<std::string, SM_State*>     
          StatePair;

  // The following typedefs are for holding the state transition information
  // based on incoming Events.
  typedef std::map<SM_EVENTS, std::string>
          TransitionStep;
          
  typedef std::pair<SM_EVENTS, std::string>
          TransitionStepPair;
          
  typedef std::map<std::string, TransitionStep>
          Transitions;

  typedef std::pair<std::string, TransitionStep>
          TransitionsPair;

  //-----------------------------------------------------------------------
  // Holds pointers to all the states currently allocated.
  States m_states;
  
  // Holds from each state to all other permitted states.
  Transitions m_transitions;
  
  // The current state for the state machine
  std::string m_currentState;

};  // SM_StateChart
  
//-------------------------------------------------------------------------
#endif

